@@ -188,10 +188,10 @@ describe Puppet::Resource::Catalog::Puppetdb do
 
       describe "for non-isomorphic resources" do
         let(:resource) do
-          Puppet::Resource.new(:exec, 'an_exec', :parameters => {:command => '/bin/true'})
+          Puppet::Resource.new(:exec, 'an_exec', :parameters => {:command => '/bin/true', :alias => 'something awesome'})
         end
 
-        it "should not create aliases" do
+        it "should not add a namevar alias" do
           hash = subject.add_parameters_if_missing(catalog_data_hash)
           result = subject.add_namevar_aliases(hash, catalog)
 
@@ -200,7 +200,7 @@ describe Puppet::Resource::Catalog::Puppetdb do
           end
 
           resource.should_not be_nil
-          resource['parameters']['alias'].should be_nil
+          resource['parameters']['alias'].should == ['something awesome']
         end
       end
     end
@@ -284,7 +284,7 @@ describe Puppet::Resource::Catalog::Puppetdb do
 
         hash = catalog.to_pson_data_hash['data']
         subject.add_parameters_if_missing(hash)
-        result = subject.synthesize_edges(hash)
+        result = subject.synthesize_edges(hash, catalog)
 
         edge = {'source' => {'type' => 'Notify', 'title' => 'anyone'},
                 'target' => {'type' => 'Notify', 'title' => 'noone'},
@@ -293,28 +293,200 @@ describe Puppet::Resource::Catalog::Puppetdb do
         result['edges'].should include(edge)
       end
 
-      it "should properly add edges for defined types" do
-        Puppet[:code] = <<-CODE
-define foo::bar() {
-  notify { $name: }
-}
-foo::bar { foo:
-  require => Foo::Bar[bar],
-}
-foo::bar { bar: }
-        CODE
+      it "should add edges from relationship arrows" do
+        other_resource = Puppet::Resource.new(:notify, 'noone')
+        Puppet[:code] = [resource, other_resource].map(&:to_manifest).join
+        Puppet[:code] << "Notify[anyone] -> Notify[noone]"
 
         hash = catalog.to_pson_data_hash['data']
         subject.add_parameters_if_missing(hash)
-        result = subject.synthesize_edges(hash)
+        result = subject.synthesize_edges(hash, catalog)
 
-        edge = {'source' => {'type' => 'Foo::Bar', 'title' => 'bar'},
-                'target' => {'type' => 'Foo::Bar', 'title' => 'foo'},
-                'relationship' => 'required-by'}
+        edge = {'source' => {'type' => 'Notify', 'title' => 'anyone'},
+                'target' => {'type' => 'Notify', 'title' => 'noone'},
+                'relationship' => 'before'}
 
         result['edges'].should include(edge)
       end
 
+      describe "exported resources" do
+        before :each do
+          Puppet[:storeconfigs] = true
+          Puppet[:storeconfigs_backend] = 'puppetdb'
+          Puppet::Resource.indirection.stubs(:search).returns []
+        end
+
+        let(:edge) do
+          {
+            'source' => {'type' => 'Notify', 'title' => 'source'},
+            'target' => {'type' => 'Notify', 'title' => 'target'},
+            'relationship' => 'before'
+          }
+        end
+
+        it "should add edges which refer to collected exported resources" do
+          Puppet[:code] = <<-MANIFEST
+          notify { source:
+            before => Notify[target],
+          }
+
+          @@notify { target: }
+
+          Notify <<| |>>
+          MANIFEST
+
+          result = subject.munge_catalog(catalog)
+
+          result['data']['edges'].should include(edge)
+        end
+
+        it "should add edges defined on collected exported resources" do
+          Puppet[:code] = <<-MANIFEST
+          @@notify { source:
+            before => Notify[target],
+          }
+
+          notify { target: }
+
+          Notify <<| |>>
+          MANIFEST
+
+          result = subject.munge_catalog(catalog)
+
+          result['data']['edges'].should include(edge)
+        end
+
+        it "should fail if an edge refers to an uncollected exported resource" do
+          Puppet[:code] = <<-MANIFEST
+          notify { source:
+            before => Notify[target],
+          }
+
+          @@notify { target: }
+          MANIFEST
+
+          expect do
+            subject.munge_catalog(catalog)
+          end.to raise_error("Invalid relationship: Notify[source] { before => Notify[target] }, because Notify[target] is exported but not collected")
+        end
+
+        it "should not add edges defined on an uncollected exported resource" do
+          Puppet[:code] = <<-MANIFEST
+          @@notify { source:
+            before => Notify[target],
+          }
+
+          notify { target: }
+          MANIFEST
+
+          result = subject.munge_catalog(catalog)
+
+          result['data']['edges'].should_not include(edge)
+        end
+      end
+
+      describe "virtual resources" do
+        let(:edge) do
+          {
+            'source' => {'type' => 'Notify', 'title' => 'source'},
+            'target' => {'type' => 'Notify', 'title' => 'target'},
+            'relationship' => 'before'
+          }
+        end
+
+        it "should add edges which refer to collected virtual resources" do
+          Puppet[:code] = <<-MANIFEST
+          notify { source:
+            before => Notify[target],
+          }
+
+          @notify { target: }
+
+          Notify <| |>
+          MANIFEST
+
+          result = subject.munge_catalog(catalog)
+
+          result['data']['edges'].should include(edge)
+        end
+
+        it "should add edges defined on collected virtual resources" do
+          Puppet[:code] = <<-MANIFEST
+          @notify { source:
+            before => Notify[target],
+          }
+
+          notify { target: }
+
+          Notify <| |>
+          MANIFEST
+
+          result = subject.munge_catalog(catalog)
+
+          result['data']['edges'].should include(edge)
+        end
+
+        it "should add edges which refer to realized virtual resources" do
+          Puppet[:code] = <<-MANIFEST
+          notify { source:
+            before => Notify[target],
+          }
+
+          @notify { target: }
+
+          realize Notify[target]
+          MANIFEST
+
+          result = subject.munge_catalog(catalog)
+
+          result['data']['edges'].should include(edge)
+        end
+
+        it "should add edges defined on realized virtual resources" do
+          Puppet[:code] = <<-MANIFEST
+          @notify { source:
+            before => Notify[target],
+          }
+
+          notify { target: }
+
+          realize Notify[source]
+          MANIFEST
+
+          result = subject.munge_catalog(catalog)
+
+          result['data']['edges'].should include(edge)
+        end
+
+        it "should fail if an edge refers to an uncollected virtual resource" do
+          Puppet[:code] = <<-MANIFEST
+          notify { source:
+            before => Notify[target],
+          }
+
+          @notify { target: }
+          MANIFEST
+
+          expect do
+            subject.munge_catalog(catalog)
+          end.to raise_error("Invalid relationship: Notify[source] { before => Notify[target] }, because Notify[target] doesn't seem to be in the catalog")
+        end
+
+        it "should not add edges defined on an uncollected virtual resource" do
+          Puppet[:code] = <<-MANIFEST
+          @notify { source:
+            before => Notify[target],
+          }
+
+          notify { target: }
+          MANIFEST
+
+          result = subject.munge_catalog(catalog)
+
+          result['data']['edges'].should_not include(edge)
+        end
+      end
+
       it "should add edges even if the other end is an alias" do
         other_resource = Puppet::Resource.new(:notify, 'noone', :parameters => {:alias => 'another_thing'})
         resource[:require] = 'Notify[another_thing]'
@@ -323,7 +495,7 @@ foo::bar { bar: }
         hash = catalog.to_pson_data_hash['data']
         subject.add_parameters_if_missing(hash)
         subject.add_namevar_aliases(hash, catalog)
-        result = subject.synthesize_edges(hash)
+        result = subject.synthesize_edges(hash, catalog)
 
         edge = {'source' => {'type' => 'Notify', 'title' => 'noone'},
                 'target' => {'type' => 'Notify', 'title' => 'anyone'},
@@ -332,40 +504,39 @@ foo::bar { bar: }
         result['edges'].should include(edge)
       end
 
-      it "should not add edges from exported resources" do
-        other_resource = Puppet::Resource.new(:notify, 'noone')
-        resource[:require] = 'Notify[noone]'
-        Puppet[:code] = "@@#{resource.to_manifest}\n#{other_resource.to_manifest}"
+      it "should produce a reasonable error message for a missing 'before' relationship" do
+        resource[:before] = 'Notify[non-existent]'
 
-        hash = catalog.to_pson_data_hash['data']
-        subject.add_parameters_if_missing(hash)
-        result = subject.synthesize_edges(hash)
-
-        edge = {'source' => {'type' => 'Notify', 'title' => 'noone'},
-                'target' => {'type' => 'Notify', 'title' => 'anyone'},
-                'relationship' => 'required-by'}
-
-        result['edges'].should_not include(edge)
-      end
-
-      it "should complain if a non-exported resource has a relationship with an exported resource" do
-        other_resource = Puppet::Resource.new(:notify, 'noone')
-        other_resource[:before] = "Notify[anyone]"
-        Puppet[:code] = "@@#{resource.to_manifest}\n#{other_resource.to_manifest}"
-
-        hash = catalog.to_pson_data_hash['data']
-        subject.add_parameters_if_missing(hash)
+        hash = subject.add_parameters_if_missing(catalog_data_hash)
         expect {
-          subject.synthesize_edges(hash)
-        }.to raise_error(/Can't create an edge between Notify\[noone\] and exported resource Notify\[anyone\]/)
+          subject.synthesize_edges(hash, catalog)
+        }.to raise_error("Invalid relationship: Notify[anyone] { before => Notify[non-existent] }, because Notify[non-existent] doesn't seem to be in the catalog")
       end
 
-      it "should complain if a resource has a relationship with a non-existent resource" do
+      it "should produce a reasonable error message for a missing 'required-by' relationship" do
         resource[:require] = 'Notify[non-existent]'
         hash = subject.add_parameters_if_missing(catalog_data_hash)
         expect {
-          subject.synthesize_edges(hash)
-        }.to raise_error("Can't synthesize edge: Notify\[anyone\] -required-by- Notify\[non-existent\] (param require)")
+          subject.synthesize_edges(hash, catalog)
+        }.to raise_error("Invalid relationship: Notify[anyone] { require => Notify[non-existent] }, because Notify[non-existent] doesn't seem to be in the catalog")
+      end
+
+      it "should produce a reasonable error message for a missing 'notifies' relationship" do
+        resource[:notify] = 'Notify[non-existent]'
+
+        hash = subject.add_parameters_if_missing(catalog_data_hash)
+        expect {
+          subject.synthesize_edges(hash, catalog)
+        }.to raise_error("Invalid relationship: Notify[anyone] { notify => Notify[non-existent] }, because Notify[non-existent] doesn't seem to be in the catalog")
+      end
+
+      it "should produce a reasonable error message for a missing 'subscription-of' relationship" do
+        resource[:subscribe] = 'Notify[non-existent]'
+
+        hash = subject.add_parameters_if_missing(catalog_data_hash)
+        expect {
+          subject.synthesize_edges(hash, catalog)
+        }.to raise_error("Invalid relationship: Notify[anyone] { subscribe => Notify[non-existent] }, because Notify[non-existent] doesn't seem to be in the catalog")
       end
     end
 
@@ -383,6 +554,64 @@ foo::bar { bar: }
 
         result['data']['edges'].should include(edge)
       end
+
+      context "when dealing with file resources and trailing slashes in their titles" do
+
+        def test_file_require(resource_title, require_title)
+          other_resource = Puppet::Resource.new(:file, resource_title)
+          resource[:require] = "File[#{require_title}]"
+          Puppet[:code] = [resource, other_resource].map(&:to_manifest).join
+          result = subject.munge_catalog(catalog)
+
+          edge = {'source' => {'type' => 'File', 'title' => resource_title},
+                  'target' => {'type' => 'Notify', 'title' => 'anyone'},
+                  'relationship' => 'required-by'}
+
+          result['data']['edges'].should include(edge)
+        end
+
+        it "should make an edge if the other end is a file resource with a missing trailing slash" do
+          test_file_require('/tmp/foo/', '/tmp/foo')
+        end
+
+        it "should make an edge if the other end is a file resource with an extra trailing slash" do
+          test_file_require('/tmp/foo', '/tmp/foo/')
+        end
+
+        it "should make an edge if the other end is a file resource with two missing trailing slashes" do
+          test_file_require('/tmp/foo//', '/tmp/foo')
+        end
+
+        it "should make an edge if the other end is a file resource with two extra trailing slashes" do
+          test_file_require('/tmp/foo', '/tmp/foo//')
+        end
+      end
+
+      it "should make an edge if the other end is an exec referred to by an alias" do
+        other_resource = Puppet::Resource.new(:exec, 'noone', :parameters => {:alias => 'completely_different', :path => '/anything'})
+        resource[:require] = 'Exec[completely_different]'
+        Puppet[:code] = [resource, other_resource].map(&:to_manifest).join
+
+        result = subject.munge_catalog(catalog)
+
+        edge = {'source' => {'type' => 'Exec', 'title' => 'noone'},
+                'target' => {'type' => 'Notify', 'title' => 'anyone'},
+                'relationship' => 'required-by'}
+
+        result['data']['edges'].should include(edge)
+      end
+
+      it "should not include virtual resources" do
+        Puppet[:code] = <<-MANIFEST
+        @notify { something: }
+        MANIFEST
+
+        result = subject.munge_catalog(catalog)
+
+        result['data']['resources'].each do |res|
+          [res['type'], res['title']].should_not == ['Notify', 'something']
+        end
+      end
     end
   end
 end
